///////////////////////////////////////////////////////////////////////////////////////////
// program which finds the longest common subsequence of exact pattern matchings (LCS-EPM)
// EPM-finding algorithm by Rolf Backofen/Sven Siebert (2007)
// LCS-EPM algorithm  by Steffen Heyne/Sebastian Will/Rolf Backofen (2008)
///////////////////////////////////////////////////////////////////////////////////////////

#include"mcsEnsemble.h"

///////////////////////////////////////////////////////////////////////////////////////////
// help screen
///////////////////////////////////////////////////////////////////////////////////////////
void printHelp()
{
  cerr << endl;
  cerr << " (c) copyright by Steffen Heyne - 2008"<< endl;
  cerr << "  - The Longest Common Subsequence of Exact Pattern Matchings (LCS-EPM) algorithm"<< endl;
  cerr << "  - (+ (EXPERIMANTAL!!!) a local clustering strategy for Exact Pattern Matchings)" << endl;
  cerr << endl << endl;
  cerr << "Usage : 1) expaRNA <options> seq1 ACGUACGU \"((....))\" seq2 ACGUACGU \"(......)\"" << endl;
  cerr << "        2) expaRNA <options> seq1 100 rand seq2 90 rand"                            << endl;
  cerr << "        3) expaRNA <options> *.mcs "                                      << endl;
  cerr << "        3) expaRNA <options> *.mcs *.align"                                      << endl << endl;
  cerr << "   1) directly given sequence and structure in dot-bracket format."   << endl;
  cerr << "      No pseudoknots supported!"                                      << endl << endl;
  cerr << "   2) generate random sequences of given length and fold it to"       << endl;
  cerr << "      a mfe-structure with Vienna RNAfold."                           << endl <<endl;
  cerr << "   3) sequence/structure data is given in file *.mcs"                 << endl;
  cerr << "      if no structure data is available, the sequence is folded"     << endl;
  cerr << "      to an mfe structure with Vienna RNAfold."                       << endl <<endl;
  cerr << "   4) like 3) and *.align contains reference alignment (use option -A)"<< endl <<endl;
  cerr << "Main Options:"                                                         << endl << endl;
  cerr << "  -L    : Calculate the maximal common pattern arrangment (LCS-EPM)"  << endl << endl;
  cerr << "  -C    : starts the local clustering algorithm with specific clustering options" << endl <<endl;
  cerr << "  -A    : determine similarity/differences of LCS-EPM with reference alignment"  << endl;
  cerr << "          via two colored postscript files"       << endl<<endl;
  cerr << "EPM Selection Options:"                                                             << endl << endl;
  cerr << "  -s#   : # minmal size of a pattern (gamma)"   << endl;
  cerr << "          2 is default and returns all patterns with at least 2 nucleotides"<< endl << endl;;
  cerr << "  -n#   : # determines the maximal number of returned patterns"       << endl;
  cerr << "          0 is default an returns all patterns"                       << endl;
  cerr << "          1 is the maximum common substructure and so on..."          << endl << endl;
  cerr << "Clustering Options: (under development!!!)"                                                             << endl << endl;
  cerr << "  -t#   : the distance-treshold value for the clustering algorithm"         << endl << endl;
  cerr << "  -c#   : # selects the clustering methode:"       << endl;
  cerr << "          0 CLUSTER_MAX_1 - cluster pattern only to best cluster before (default)"<< endl;
  cerr << "          1 CLUSTER_MAX_2 - like 0, but best cluster could not be used again!"<< endl;
  cerr << "          2 CLUSTER_ALL   - cluster pattern to all clusters within t"   << endl << endl;
  cerr << "  -d#   : # selects the  distance methode:"       << endl;
  cerr << "          0 DIST_SEQUENCE     - shortest sequence between 2 patterns (defualt)" << endl;
  cerr << "          1 DIST_EQUAL        - distances do not differ more than q#" << endl << endl;
  cerr << "          2 DIST_SHORTESTPATH - shortest path (!) through the structure"  << endl << endl;
  cerr << "  -q#   : the distance-equal threshold for -d1" << endl << endl;
  cerr << "Output Options:"                                                             << endl << endl;
  cerr << "  -p#   : save biggest # cluster as colored postscript file (1 for LC-ERP)"                  << endl << endl;
  cerr << "  -x    : save data to xml file"                            << endl << endl;
  cerr << "  -h    : this info"                                                  << endl << endl;
}

///////////////////////////////////////////////////////////////////////////////////////////
// main
///////////////////////////////////////////////////////////////////////////////////////////
int main(int argc,char* argv[])
{
  ///////////////////////////////////////////////////////////////////////////////
  // read command line options
  ///////////////////////////////////////////////////////////////////////////////

  int    		opt;
  mcsOptions	myOptions;

  while ((opt=getopt(argc,argv,"s:n:LCAc:d:t:q:p:xh"))!=-1)
   switch(opt)
   {
      case 's':
            myOptions.minPatternSize = atoi(optarg);
            if (myOptions.minPatternSize<0)
            {
               cerr << endl << " Minimal pattern size not valid!" << endl << endl;
               exit(1);
            }
         break;
      case 'n':
		     myOptions.maxPatternNumber = atoi(optarg);
			    if (myOptions.maxPatternNumber<0)
             {
               cerr << endl << " The number of returned patterns is not valid!" << endl << endl;
	    			exit(1);
             }
         break;
      case 'L':
      		myOptions.calcERPAP = true;
      	break;
      case 'C':
           	myOptions.clustering = true;
        break;
      case 'A':
                 	myOptions.alignColor = true;
              break;
      case 'c':
		     myOptions.CLUSTER_METHODE = static_cast<mcsOptions::CLUSTER_METHODE_Options>(atoi(optarg));
			    if ((myOptions.CLUSTER_METHODE < 0) || (myOptions.CLUSTER_METHODE > 2))
             {
               cerr << endl << " This cluster methode is not valid!" << endl << endl;
	    			exit(1);
             }
         break;
     	case 'd':
		     myOptions.DISTANCE_METHODE = static_cast<mcsOptions::DISTANCE_METHODE_Options>(atoi(optarg));
			    if ((myOptions.DISTANCE_METHODE < 0) || (myOptions.DISTANCE_METHODE > 1))
             {
               cerr << endl << " This distance methode is not valid!" << endl << endl;
	    			exit(1);
             }
         break;
      case  't':
            myOptions.distanceThresholdClustering = atoi(optarg);
            if (myOptions.distanceThresholdClustering<1)
             {
               cerr << endl << " The distance threshold for clustering is <1 !!!" << endl << endl;
               exit(1);
             }
         break;
      case  'q':
                  myOptions.distanceEqualThreshold = atoi(optarg);
                  if (myOptions.distanceEqualThreshold<1)
                   {
                     cerr << endl << " The distance Equal threshold for clustering is <1 !!!" << endl << endl;
                     exit(1);
                   }
               break;
      case 'p':
            myOptions.clusterSaveCounter = atoi(optarg);
            if (myOptions.clusterSaveCounter<1)
             {
               cerr << endl << " Please specify a number >=1 !!!" << endl << endl;
               exit(1);
             }
         break;
      case 'x':
            myOptions.writeXML = true;
         break;
		case 'h':
		       printHelp();
			    exit(0);
         break;
      	case '?' :
             cerr << "Unrecognized option." << endl;
			    printHelp();
			    exit(1);
	     break;
  	}

  	///////////////////////////////////////////////////////////////////////////////
  	// Input recognition
  	///////////////////////////////////////////////////////////////////////////////

	for(;optind<argc;optind++)
		myOptions.cmdLineValues.push_back(argv[optind]);

  	int size=myOptions.cmdLineValues.size();
  	if ((size!=4) && (size!=6) && (size!=1)&& (size!=2))
   	{
    	cerr << "Please use correct input format!" << endl;
    	printHelp();
		exit(1);
   }
  	///////////////////////////////////////////////////////////////////////////////
   //  load data into the main interface and generate the two molecules
   ///////////////////////////////////////////////////////////////////////////////
   mcsEnsemble myNewEnsemble(myOptions);

   ///////////////////////////////////////////////////////////////////////////////
   //  start the maximum common substructure algorithm with the provided options
   //  start the clustering algorithm for these patterns and provided options
   ///////////////////////////////////////////////////////////////////////////////
   myNewEnsemble.startALL();

   return(0);
};
