#ifndef MCSCLUSTERING_H_
#define MCSCLUSTERING_H_


#include<iomanip>
#include<climits>
#include<math.h>

#include "DATA_Patterns.h"
//#include <boost/weak_ptr.hpp>
#include "mcsOptions.h"


//-----------------------------------------------------------------------------------------------------------
// class ClusterMolecule is an extension of class Molecule
//
//    provides things necessary for Clustering of patterns
//    main types:
//                 sequence[]   vector with information about nucleotides   
//-----------------------------------------------------------------------------------------------------------
class ClusterMolecule: public Molecule
{
	public:
		//---------------------------------------------------------------------------
		// type definitions
		//--------------------------------------------------------------------------- 
		enum PairType   	{ UNPAIRED, LEFTPAIRED, RIGHTPAIRED, UNDEFPAIR };
        enum StructureType	{ LOOP, STACKING, STEM_OUTSIDE_MIN, STEM_OUTSIDE_MAX, STEM_INSIDE_END, UNDEFSTRUCTURE };
          
        struct Nucleotide
        {
        	PairType		pairType;     
            StructureType  	structureType;
            unsigned int   	position;
            int            	connectedTo;  
            intPair        	lastClosingBond;          
         };
         typedef Nucleotide*                       NucleotidePTR;
         //typedef boost::shared_ptr<Nucleotide>   NucleotidePTR;
         typedef vector<NucleotidePTR>             SequenceTYPE;
         typedef SequenceTYPE::iterator            SequenceITER; 
         
      //---------------------------------------------------------------------------
      // constructors/destructors
      //---------------------------------------------------------------------------
         			ClusterMolecule   (const Molecule& myMol):Molecule(myMol){};
         virtual	~ClusterMolecule  () {};
      
      //---------------------------------------------------------------------------
      // public member definitions
      //---------------------------------------------------------------------------                
                  SequenceTYPE     sequence; 
            
      //---------------------------------------------------------------------------
      // public member methods
      //---------------------------------------------------------------------------                     
						void           	initSequence   ();
         
	private:
      //---------------------------------------------------------------------------
      // private member definitions
      //---------------------------------------------------------------------------      
   
      //---------------------------------------------------------------------------
      // private methods
      //---------------------------------------------------------------------------                
               	
      	const 				PairType  		convertStructure  (const char& 			myStr);
};


////////////////////////////////////////////////////////////////////////////////////
// class mcsClustering
//    the main class for clustering algo
////////////////////////////////////////////////////////////////////////////////////
class mcsClustering
{
	public:
	
	   //---------------------------------------------------------------------------
	   // type definitions
	   //--------------------------------------------------------------------------- 
		
   		enum 		CLUSTER_METHODE_E 	{ CLUSTER_MAX_1, CLUSTER_MAX_2, CLUSTER_ALL };
   		enum 		DISTANCE_METHODE_E	{ DIST_SEQUENCE, DIST_EQUAL, DIST_SHORTESTPATH };
   		enum		PatternType_E    	{ LOOPPATTERN, STEMPATTERN, STEMLOOPPATTERN, HAIRPINPATTERN };
   		enum 		ClusterNucleotide_E	{ CLUSTER_NOTHING_BASE, CLUSTER_TO_BEFORE_BASE, CLUSTER_STEM_TO_BEFORE_BASE, CLUSTER_TO_LOOP_BASE};
			
		// types fromn enumerations 
   		typedef 	PatternType_E									PatternType;
   		typedef 	ClusterNucleotide_E                 			ClusterNucleotideType;
   		
   		// types from PatternKey: position=outsideMaxBound, all data from a pattern like id, min/max, boundaries etc.
   		// PatternKeyList is list of all pattern ending at the same position
   		struct 		PatternKeyS;
   		typedef		PatternKeyS										PatternKey;
   		//typedef 	boost::shared_ptr<PatternKey>   				PatternKeyPTR;
   		typedef 	PatternKey*                       				PatternKeyPTR;
   		typedef 	vector<PatternKeyPTR>             				PatternKeyListTYPE;
   		typedef 	PatternKeyListTYPE::const_iterator  			PatternKeyListCITER;
   		
   		// types from ClusterInfoS: all data for a cluster, a cluster is build up from patternKeys
   		// ClusterList is for all Clusters which are "active" at one nucleotide
   		// ClusterMap is for traceback to save all found clusters, map is ordered by clusterSize	
   		struct  	ClusterInfoS;
   		typedef 	ClusterInfoS                        			ClusterInfoTYPE;
        //typedef 	boost::shared_ptr<ClusterInfoTYPE>  			ClusterInfoPTR;
        typedef 	ClusterInfoTYPE*                    			ClusterInfoPTR;
         
        typedef		vector<ClusterInfoPTR>              			ClusterListTYPE;
        typedef 	ClusterListTYPE::const_iterator     			ClusterListCITER;
        typedef 	ClusterListTYPE::iterator           			ClusterListITER;
        
        typedef  	multimap<int,ClusterInfoPTR,greater<int> >   	ClusterMapTYPE;
        typedef  	ClusterMapTYPE::const_iterator               	ClusterMapCITER;
        typedef  	ClusterMapTYPE::const_iterator               	ClusterMapITER;   
        
        // types from ClusterNucleotide: all data for the main algo which goes through the sequence of ClusterNucleotides
        struct		ClusterNucleotideS;
        typedef 	ClusterNucleotideS								ClusterNucleotide;
        typedef		ClusterNucleotide*								ClusterNucleotidePTR;
        typedef 	vector<ClusterNucleotidePTR>					ClusterSequenceTYPE;
        typedef 	ClusterSequenceTYPE::const_iterator				ClusterSequenceCITER ;
        typedef 	ClusterSequenceTYPE::iterator      				ClusterSequenceITER; 
         
   		struct PatternKeyS
   		{
   			PatternType patternType;
		    string      patternId;
		    int         position;
			int         patternSize;      
      		intVec      allBounds;
      		intVec      allBoundsREF;
      		intVec      minBounds;
      		intVec      maxBounds;
      		intVec      minBoundsREF;
      		intVec      maxBoundsREF; 
      		intPair     minmax;
      		intPair     minmaxREF;
      		intPair     lastClosingBond;
      		intPair     lastClosingBondREF;
   		}; 
   		
   		struct ClusterInfoS
         {
            string                  clusterId;
            ClusterListTYPE         lastClusterList;		//the list of clusters 
            PatternKeyPTR           newPattern;				//the last added pattern to the cluster
            PatternKeyListTYPE		clusterPatternList;		//patterns of the cluster
            intPair                 clusterMinMax;			//nucleotide position of min/max
            PatternKeyPTR           clusterMinKey;			//pointer to pattern at min
            PatternKeyPTR           clusterMaxKey;			//pointer to pattern at max
            int                     clusterSize;			//size in nucleotides of cluster
            int                     actDistance;			//counter for distance: last cluster pattern to actual position in strand
            int                     weight;					//not used so far
            bool                    visited;				//for traceback
            bool					clustered;				//for CLUSTER_MAX_2
         };
         
         struct ClusterNucleotideS
         {
            ClusterMolecule::NucleotidePTR		nucleotide;
            PatternKeyListTYPE					patternKeyList;
            ClusterNucleotideType            	clusterNucleotideType;
            ClusterListTYPE                  	clusterList;
            ClusterNucleotidePTR				beforeNuc;
            ClusterNucleotidePTR				afterNuc;
            ClusterNucleotidePTR				connectedToNuc;
         };
         
      //---------------------------------------------------------------------------
      // constructors/destructors
      //---------------------------------------------------------------------------
                     mcsClustering(const Molecule&       myMol1,
                                   const Molecule&       myMol2,
                                   const PatternPairMap& myPatterns,
                                         ClusterMapTYPE& myClusters,
                                         mcsOptions		 myOptions)
                                   
                                   :DISTANCE_THRESHOLD(myOptions.distanceThresholdClustering),
                                    DISTANCE_EQUAL_THRESHOLD(myOptions.distanceEqualThreshold),
                                   	CLUSTER_METHODE((static_cast<CLUSTER_METHODE_E>(myOptions.CLUSTER_METHODE))),
                                   	DISTANCE_METHODE((static_cast<DISTANCE_METHODE_E>(myOptions.DISTANCE_METHODE))),
                                    mol1(myMol1),mol2(myMol2),
                                   	patterns(myPatterns),
                                    clusters(myClusters),
                                    minPatternSize(myOptions.minPatternSize) {};

         virtual     ~mcsClustering() {};
      
      //---------------------------------------------------------------------------
      // public member definitions
      //---------------------------------------------------------------------------                     
                  
      //---------------------------------------------------------------------------
      // public member methods
      //---------------------------------------------------------------------------
               void  			calculateClusters();
          
   private:
      //---------------------------------------------------------------------------
      // private member definitions
      //--------------------------------------------------------------------------- 
            
			ClusterSequenceTYPE	clusterSequence;
				
			int               	DISTANCE_THRESHOLD;
			int 				DISTANCE_EQUAL_THRESHOLD;
            CLUSTER_METHODE_E  	CLUSTER_METHODE;
            DISTANCE_METHODE_E	DISTANCE_METHODE;
            
            ClusterMolecule	   	mol1,mol2;
      const PatternPairMap&     patterns;
            ClusterMapTYPE&     clusters;

            int                 minPatternSize;
            int                 clusterCounter;
            
   //---------------------------------------------------------------------------
   // private member methods
   //---------------------------------------------------------------------------            
            void              	clustering();
            void              	preProcessing();
            
            //void				calculatePatternBoundaries	(const boost::shared_ptr<PatternPair>& myPair,
                            //                           			PatternKeyPTR myKey);
                            
            void calculatePatternBoundaries	(const PatternPair* myPair,
                                         			PatternKeyPTR myKey);
                                        
            
            void              	updateClusterList(const ClusterNucleotidePTR& toPos);
            void              	clusterActPatterns(const ClusterNucleotidePTR& myPos);
            void 				clusterToCluster(const ClusterNucleotidePTR& posStemEnd);
            int               	distance(   const ClusterInfoPTR& cluster1,
            								const ClusterInfoPTR& cluster2,
            								const int testBound,
            								bool& crossing, bool& overlapping, bool isPosBeforeTest);

            void              	collectClusters();
            void              	traverseCluster(ClusterInfoPTR myClus);
            
            void              	printPatInfo(const SinglePattern& myPat1,const SinglePattern& myPat2); 
            void              	printCluster(const ClusterInfoPTR myClus);
  //        void              	printClusterList(const BaseClusterListTYPE myList,bool printListDetails);
            void              	printClusterInfo();  
};

#endif /*MCSCLUSTERING_H_*/
